package com.example.testapi.controller;

import com.example.testapi.model.Candle;
import com.example.testapi.model.DateRange;
import com.example.testapi.model.Price;
import com.example.testapi.service.ApiService;
import com.example.testapi.service.SimulateurService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api")
public class ApiController {

    private final ApiService apiService;

    private final SimulateurService simulateur;

    private DateRange dateRange;

    @Autowired
    public ApiController(ApiService apiService, SimulateurService simulateur) {
        this.apiService = apiService;
        this.simulateur = simulateur;
    }

    @GetMapping("/btcusdt")
    public ResponseEntity<?> getCandlesData() throws InterruptedException, com.fasterxml.jackson.core.JsonProcessingException {
        System.out.println("GET /btcusdt");
        List<Candle> candles = apiService.getCandles(dateRange.getTimestampStartDate(), dateRange.getTimestampEndDate());
        if (candles == null) {
            return ResponseEntity.status(HttpStatus.SERVICE_UNAVAILABLE).body("Impossible de récupérer les données de l'API Binance.");
        }
        return ResponseEntity.ok(candles);
    }

    @PostMapping("/sendDates")
    public Map<String, Object> sendDates(@RequestBody DateRange json) throws InterruptedException, com.fasterxml.jackson.core.JsonProcessingException {
        System.out.println("POST /sendDates");
        System.out.println("Dates received: " + json.getStartDate() + " - " + json.getEndDate());
        this.dateRange = new DateRange(json.getStartDate(), json.getEndDate());
        return Collections.emptyMap();
    }

//    @GetMapping("/startSimu")
//    public void testSomeFonctionnalities() {
//        this.simulateur.startSimulation();
//        long[] x_tab = {};
//        double[] y = new double[4];
//        this.simulateur.approximatePrice(x_tab, y, 3);
//    }

    @GetMapping("/prices")
    public List<Price> getPrices() {
        System.out.println("GET /prices");
        return this.simulateur.getPricesForPeriod(this.dateRange);
    }

//    @GetMapping("/stopSimu")
//    public void stopSimulation() {
//        this.simulateur.stopSimulation();
//    }

    @GetMapping("/test")
    public void test() {
        long[] x_tab = {1, -1, 2};
        double[] y = {3, 2, -1};
        this.simulateur.approximatePrice(x_tab, y, 2);
    }
}

