package com.example.testapi.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.example.testapi.model.Candle;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class ApiService {
    public String hello() {
        return "Hello World";
    }

    @Autowired
    private final RestTemplate restTemplate;

    private final SimulateurService simulateurService;

    private static final String baseUrl = "https://api4.binance.com/api/v3/klines";

    private static final long TIME_10_JOURS = 10 * 24 * 60 * 60 * 1000;

    @Autowired
    public ApiService(SimulateurService simulateurService, RestTemplate restTemplate) {
        this.simulateurService = simulateurService;
        this.restTemplate = restTemplate;
    }

    public List<Candle> getCandles() {
        return this.simulateurService.getCandles();
    }

    private Optional<List<Candle>> getCandlesFromAPI(long startTime, long endTime) throws InterruptedException, com.fasterxml.jackson.core.JsonProcessingException {
        if (endTime < startTime) {
            System.out.println("La date de fin ne peut pas être avant la date de début");
            return Optional.empty();
        }
        System.out.println("in");
        try {
            System.out.println("in try");
            List<List<Object>> jsonCandles = null;
            System.out.println("debut : "+ startTime);
            System.out.println("fin : "+ endTime);
            List<Candle> candles = new ArrayList<>();
            while (startTime < endTime)
            {
                long currEndTime = Math.min(startTime + TIME_10_JOURS + 1, endTime);
                String url = UriComponentsBuilder.fromHttpUrl(baseUrl)
                        .queryParam("symbol", "BTCUSDT")
                        .queryParam("interval", "15m")
                        .queryParam("startTime", startTime)
                        .queryParam("endTime", currEndTime)
                        .queryParam("limit", 960)
                        .toUriString();
                String response = restTemplate.getForObject(url, String.class);

                ObjectMapper objectMapper = new ObjectMapper();
                jsonCandles = objectMapper.readValue(response, new TypeReference<List<List<Object>>>() {});
                for (List<Object> jsonCandle : jsonCandles) {
                    candles.add(new Candle(
                            Long.parseLong(jsonCandle.get(0).toString()),
                            Double.parseDouble(jsonCandle.get(1).toString()),
                            Double.parseDouble(jsonCandle.get(2).toString()),
                            Double.parseDouble(jsonCandle.get(3).toString()),
                            Double.parseDouble(jsonCandle.get(4).toString()),
                            Long.parseLong(jsonCandle.get(6).toString())
                    ));
                }
                // avancer de 1000 * 15 minutes * 60 pour convertir en secondes et * 1000 pour millisecondes
                startTime += TIME_10_JOURS + 1;
            }
            return Optional.of(candles);
        } catch (Exception e) {
            System.err.println("Erreur récupération des candles : " + e.getMessage());
            return Optional.empty();
        }
    }

    public List<Candle> getCandles(long startDate, long endDate) throws InterruptedException, com.fasterxml.jackson.core.JsonProcessingException {
        if (this.simulateurService.getCandles() == null || this.simulateurService.getCandles().isEmpty()) {
            System.out.println("init list candles");
            System.out.println("debut : "+ startDate);
            System.out.println("fin : "+ endDate);
            Optional<List<Candle>> optJsonCandles = this.getCandlesFromAPI(startDate, endDate);
            List<Candle> candles = optJsonCandles.orElse(null);
            if (candles != null) {
                this.simulateurService.setCandles(candles);                     
                System.out.println("init list candles sim");
            }
            else {
                System.out.println("null json candles");
                return null;
            }
        }
        System.out.println("return list");
        return this.simulateurService.getCandles();

    }

//    public String candlesToJson(long startDate, long endDate) throws InterruptedException, com.fasterxml.jackson.core.JsonProcessingException {
//
//    }
}
