package com.example.testapi.service;

import com.example.testapi.model.Candle;
import com.example.testapi.model.DateRange;
import com.example.testapi.model.Interval;
import com.example.testapi.model.Price;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

@Service
public class SimulateurService {
    private List<Candle> candles;
    private List<Price> prices;
    private Random rnd;

    public List<Candle> getCandles() {
        return candles;
    }

    public void setCandles(List<Candle> candles) {
        this.candles = candles;
    }

    public SimulateurService() {
        this.prices = new ArrayList<>();
        this.rnd = new Random();
    }

    public List<Price> getPrices() {
        return prices;
    }

    /**
     * Permet de faire une approximation
     * @param x_tab
     * @param y
     * @param x dans notre cas représente le "temps"
     * @return la valeur de l'approximation du prix
     */
    public Price approximatePrice(long[] x_tab, double[] y, long x) {
        double value = 0.0;
        int n = x_tab.length;
        for (int i = 0; i < n; i++) {
            double tmp = y[i];
            for (int j = 0; j < n; j++) {
                if (i != j) {
                    tmp *= (double) (x - x_tab[j]) / (x_tab[i] - x_tab[j]);
                }
            }
            value += tmp;
        }
        return new Price(x, value);
    }

    /**
     *
     * currentTimestamp la seconde actuelle
     * timestamps générer 4 timestamps random dans l'intervalle de 15 min pour OHLC
     * priceOHLC valeur de OHLC
     * @param interval
     */
    public void taskEachSec(Interval interval, long[] timestamps, double[] priceOHLC) {
        Candle currentCandle = interval.getCurrentCandle();
        long currentTimestamp = currentCandle.getTime() + interval.getDeltaTime();
        Price tmpPrice = this.approximatePrice(timestamps, priceOHLC, currentTimestamp);
        this.prices.add(new Price(tmpPrice.getTime(), interval.getCurrentCandle().checkprice(tmpPrice.getValue())));
        interval.nextSecond();
    }

    public double[] randomPositionsOHLC(Candle currentCandle) {
        double[] priceOHLC;
        if (currentCandle.getLow() == currentCandle.getOpen() && currentCandle.getHigh() == currentCandle.getClose() || currentCandle.getHigh() == currentCandle.getOpen() && currentCandle.getLow() == currentCandle.getClose()) {
            priceOHLC = new double[] {currentCandle.getOpen(), currentCandle.getClose()};
        }
        else if (currentCandle.getLow() == currentCandle.getOpen() || currentCandle.getLow() == currentCandle.getClose()) {
            priceOHLC = new double[] {currentCandle.getOpen(), currentCandle.getHigh(), currentCandle.getClose()};
        }
        else if (currentCandle.getHigh() == currentCandle.getOpen() || currentCandle.getHigh() == currentCandle.getClose()) {
            priceOHLC = new double[] {currentCandle.getOpen(), currentCandle.getLow(), currentCandle.getClose()};
        }
        else {
            priceOHLC = new double[4];
            priceOHLC[0] = currentCandle.getOpen();
            if (this.rnd.nextBoolean()) {
                priceOHLC[1] = currentCandle.getHigh();
                priceOHLC[2] = currentCandle.getLow();
            }
            else {
                priceOHLC[1] = currentCandle.getLow();
                priceOHLC[2] = currentCandle.getHigh();
            }
            priceOHLC[3] = currentCandle.getClose();
        }
        return priceOHLC;
    }

    public long[] generateRandomTimstamps(long startTimestamp, long endTimestamp, Candle currentCandle) {
        int nbTimestamps;
        if (currentCandle.getLow() == currentCandle.getOpen() && currentCandle.getHigh() == currentCandle.getClose() || currentCandle.getHigh() == currentCandle.getOpen() && currentCandle.getLow() == currentCandle.getClose()) {
            nbTimestamps = 2;
        }
        else if (currentCandle.getLow() == currentCandle.getOpen() || currentCandle.getHigh() == currentCandle.getOpen() || currentCandle.getHigh() == currentCandle.getClose() || currentCandle.getLow() == currentCandle.getClose()) {
            nbTimestamps = 3;
        }
        else {
            nbTimestamps = 4;
        }
        long rndTimestamp1 = generateRandomTimstamp(startTimestamp+1, endTimestamp-2);
        long rndTimestamp2 = generateRandomTimstamp(rndTimestamp1+1, endTimestamp-1);
        long[] timestamps;
        if (nbTimestamps == 2) {
            timestamps = new long[]{startTimestamp, endTimestamp};
        }
        else if (nbTimestamps == 3) {
            timestamps = new long[]{startTimestamp, rndTimestamp1, endTimestamp};
        }
        else if (rndTimestamp2 == endTimestamp) {
            timestamps = new long[]{startTimestamp, rndTimestamp1, endTimestamp};
        }
        else {
            timestamps = new long[]{startTimestamp, rndTimestamp1, rndTimestamp2, endTimestamp};
        }
        return timestamps;
    }

    public long generateRandomTimstamp(long startTimestamp, long endTimestamp) {
        return this.rnd.nextLong(endTimestamp - startTimestamp + 1) + startTimestamp;
    }

    public List<Price> getPricesForPeriod(DateRange period) {
        final int nbTimestamp15Min = 15 * 60;
        long endTime = period.getTimestampEndDateInSeconds();
        int indexCurrentCandle = 0;
        Candle currentCandle = candles.get(indexCurrentCandle);
        Interval interval = new Interval(currentCandle);
        long[] timestamps = generateRandomTimstamps(currentCandle.getTime(), currentCandle.getCloseTime(), currentCandle);
        double[] priceOHLC = randomPositionsOHLC(currentCandle);
        while (interval.getCurrentCandle().getTime() < endTime) {
            // Boucle qui simule 15 min
            for (int i = 0; i < nbTimestamp15Min; i++) {
                taskEachSec(interval, timestamps, priceOHLC);
            }
            indexCurrentCandle++;
            currentCandle = candles.get(indexCurrentCandle);
            interval = new Interval(currentCandle);
            timestamps = generateRandomTimstamps(currentCandle.getTime(), currentCandle.getCloseTime(), currentCandle);
            priceOHLC = randomPositionsOHLC(currentCandle);
        }
        System.out.println("len prices = " + this.prices.size());
        return this.prices;
    }
}
